'use strict';

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _server = require('react-dom/server');

var _enzyme = require('enzyme');

var _reactAsyncBootstrapper = require('react-async-bootstrapper');

var _reactAsyncBootstrapper2 = _interopRequireDefault(_reactAsyncBootstrapper);

var _ = require('../');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function Bob(_ref) {
  var children = _ref.children;

  return _react2.default.createElement(
    'div',
    null,
    children
  );
}
Bob.propTypes = { children: _react2.default.PropTypes.node };
Bob.defaultProps = { children: null };

var AsyncBob = (0, _.asyncComponent)({
  resolve: function resolve() {
    return new Promise(function (resolve) {
      return setTimeout(function () {
        return resolve(Bob);
      }, 10);
    });
  },
  name: 'AsyncBob'
});

var AsyncBobTwo = (0, _.asyncComponent)({
  resolve: function resolve() {
    return new Promise(function (resolve) {
      return setTimeout(function () {
        return resolve(Bob);
      }, 10);
    });
  },
  name: 'AsyncBobTwo'
});

var AsyncBobThree = (0, _.asyncComponent)({
  resolve: function resolve() {
    return new Promise(function (resolve) {
      return setTimeout(function () {
        return resolve(Bob);
      }, 10);
    });
  },
  name: 'AsyncBobThree'
});

var DeferredAsyncBob = (0, _.asyncComponent)({
  resolve: function resolve() {
    return new Promise(function (resolve) {
      return setTimeout(function () {
        return resolve(Bob);
      }, 10);
    });
  },
  ssrMode: 'defer',
  name: 'DeferredAsyncBob',
  LoadingComponent: function LoadingComponent() {
    return _react2.default.createElement(
      'div',
      null,
      'DeferredAsyncBob Loading'
    );
  }
});

var BoundaryAsyncBob = (0, _.asyncComponent)({
  resolve: function resolve() {
    return new Promise(function (resolve) {
      return setTimeout(function () {
        return resolve(Bob);
      }, 10);
    });
  },
  ssrMode: 'boundary',
  name: 'BoundaryAsyncBob'
});

var ErrorAsyncComponent = (0, _.asyncComponent)({
  resolve: function resolve() {
    return new Promise(function () {
      throw new Error('This always errors');
    });
  },
  name: 'ErrorAsyncComponent',
  // eslint-disable-next-line react/prop-types
  ErrorComponent: function ErrorComponent(_ref2) {
    var message = _ref2.message;
    return _react2.default.createElement(
      'div',
      null,
      message
    );
  }
});

var createApp = function createApp(asyncContext, stateForClient) {
  return _react2.default.createElement(
    _.AsyncComponentProvider,
    {
      asyncContext: asyncContext,
      rehydrateState: stateForClient
    },
    _react2.default.createElement(
      AsyncBob,
      null,
      _react2.default.createElement(
        'div',
        null,
        _react2.default.createElement(
          AsyncBobTwo,
          null,
          _react2.default.createElement(
            'span',
            null,
            'In Render.'
          )
        ),
        _react2.default.createElement(
          DeferredAsyncBob,
          null,
          _react2.default.createElement(
            'span',
            null,
            'In Defer.'
          )
        ),
        _react2.default.createElement(
          BoundaryAsyncBob,
          null,
          _react2.default.createElement(
            'span',
            null,
            'In Boundary but outside an AsyncComponent, server render me!'
          ),
          _react2.default.createElement(
            AsyncBobThree,
            null,
            _react2.default.createElement(
              'span',
              null,
              'In Boundary - Do not server render me!'
            )
          )
        ),
        _react2.default.createElement(ErrorAsyncComponent, null)
      )
    )
  );
};

describe('integration tests', function () {
  it('render server and client', function () {
    // we have to delete the window to emulate a server only environment
    var windowTemp = global.window;
    delete global.window;

    // "Server" side render...
    var serverContext = (0, _.createAsyncContext)();
    var serverApp = createApp(serverContext);
    return (0, _reactAsyncBootstrapper2.default)(serverApp).then(function () {
      var serverString = (0, _server.renderToStaticMarkup)(serverApp);
      expect(serverString).toMatchSnapshot();
      var stateForClient = serverContext.getState();
      expect(stateForClient).toMatchSnapshot();
      // Restore the window and attach the state to the "window" for the
      // client
      global.window = windowTemp;
      return { serverHTML: serverString, stateForClient: stateForClient };
    }).then(function (_ref3) {
      var serverHTML = _ref3.serverHTML,
          stateForClient = _ref3.stateForClient;

      // "Client" side render...
      var clientContext = (0, _.createAsyncContext)();
      var clientApp = createApp(clientContext, stateForClient);
      return (0, _reactAsyncBootstrapper2.default)(clientApp).then(function () {
        var clientRenderWrapper = (0, _enzyme.mount)(clientApp);
        expect(clientRenderWrapper).toMatchSnapshot();
        expect((0, _server.renderToStaticMarkup)(clientApp)).toEqual(serverHTML);
        return clientRenderWrapper;
      })
      // Now give the client side components time to resolve
      .then(function (clientRenderWrapper) {
        return new Promise(function (resolve) {
          return setTimeout(function () {
            return resolve(clientRenderWrapper);
          }, 100);
        });
      })
      // Now a full render should have occured on client
      .then(function (clientRenderWrapper) {
        return expect(clientRenderWrapper).toMatchSnapshot();
      });
    });
  });

  it('renders the LoadingComponent', function () {
    var Foo = (0, _.asyncComponent)({
      resolve: function resolve() {
        return new Promise(function (resolve) {
          return setTimeout(function () {
            return resolve(function () {
              return _react2.default.createElement(
                'div',
                null,
                'foo'
              );
            });
          }, 100);
        });
      },
      LoadingComponent: function LoadingComponent() {
        return _react2.default.createElement(
          'div',
          null,
          'Loading...'
        );
      }
    });

    var asyncContext = (0, _.createAsyncContext)();

    var app = _react2.default.createElement(
      _.AsyncComponentProvider,
      { asyncContext: asyncContext },
      _react2.default.createElement(Foo, null)
    );

    return (0, _reactAsyncBootstrapper2.default)(app).then(function () {
      return (0, _enzyme.mount)(app);
    }).then(function (render) {
      return expect(render).toMatchSnapshot();
    });
  });

  it('renders the ErrorComponent', function () {
    var Foo = (0, _.asyncComponent)({
      resolve: function resolve() {
        throw new Error('An error occurred');
      },
      // eslint-disable-next-line react/prop-types
      ErrorComponent: function ErrorComponent(_ref4) {
        var message = _ref4.message;
        return _react2.default.createElement(
          'div',
          null,
          message
        );
      }
    });

    var asyncContext = (0, _.createAsyncContext)();

    var app = _react2.default.createElement(
      _.AsyncComponentProvider,
      { asyncContext: asyncContext },
      _react2.default.createElement(Foo, null)
    );

    return (0, _reactAsyncBootstrapper2.default)(app).then(function () {
      return new Promise(function (resolve) {
        var rendered = (0, _enzyme.mount)(app);
        setTimeout(function () {
          return resolve(rendered);
        }, 100);
      });
    }).then(function (render) {
      return expect(render).toMatchSnapshot();
    });
  });

  it('a component only gets resolved once', function () {
    var resolveCount = 0;

    var Foo = (0, _.asyncComponent)({
      resolve: function resolve() {
        resolveCount += 1;
        return function () {
          return _react2.default.createElement(
            'div',
            null,
            'foo'
          );
        };
      }
    });

    var asyncContext = (0, _.createAsyncContext)();

    var app = _react2.default.createElement(
      _.AsyncComponentProvider,
      { asyncContext: asyncContext },
      _react2.default.createElement(Foo, null)
    );

    return (0, _reactAsyncBootstrapper2.default)(app).then(function () {
      return (0, _enzyme.mount)(app);
    }).then(function () {
      return expect(resolveCount).toEqual(1);
    }).then(function () {
      return (0, _enzyme.mount)(app);
    }).then(function () {
      return expect(resolveCount).toEqual(1);
    });
  });
});