'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = withAsyncComponents;

var _reactTreeWalker = require('react-tree-walker');

var _reactTreeWalker2 = _interopRequireDefault(_reactTreeWalker);

var _constants = require('./constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function withAsyncComponents(app) {
  var isBrowser = typeof window !== 'undefined';
  var rehydrateState = isBrowser && typeof window[_constants.STATE_IDENTIFIER] !== 'undefined' ? window[_constants.STATE_IDENTIFIER] : null;

  if (isBrowser && !rehydrateState) {
    return Promise.resolve();
  }

  var visitor = function visitor(element, instance, context) {
    if (instance && typeof instance.getAsyncComponentData === 'function') {
      var _instance$getAsyncCom = instance.getAsyncComponentData(),
          id = _instance$getAsyncCom.id,
          ssrMode = _instance$getAsyncCom.ssrMode,
          getResolver = _instance$getAsyncCom.getResolver;

      var isBoundary = context.asyncComponentsAncestor && context.asyncComponentsAncestor.isBoundary;

      if (rehydrateState != null) {
        if (!rehydrateState.resolved[id]) {
          return false;
        }
        rehydrateState[id] = false;
      } else if (ssrMode === 'defer' || isBoundary) {
        // Deferred, so return false to stop walking down this branch.
        return false;
      }
      return getResolver();
    }
    return true;
  };

  return (0, _reactTreeWalker2.default)(app, visitor, {})
  // Swallow errors.
  .catch(function () {
    return undefined;
  });
}